import { OPTIONS } from '@/constants/map';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import { mapDataSizeSelector } from '@/redux/map/map.selectors';
import { currentModelIdSelector } from '@/redux/models/models.selectors';
import { View } from 'ol';
import { unByKey } from 'ol/Observable';
import { useEffect, useRef } from 'react';
import { useSelector } from 'react-redux';
import { useDebouncedCallback } from 'use-debounce';
import { Pixel } from 'ol/pixel';
import { Coordinate } from 'ol/coordinate';
import { MapInstance } from '../../MapViewer.types';
import { onMapSingleClick } from './mapSingleClick/onMapSingleClick';
import { onMapRightClick } from './mapRightClick/onMapRightClick';
import { onMapPositionChange } from './onMapPositionChange';

interface UseOlMapListenersInput {
  view: View;
  mapInstance: MapInstance;
}

export const useOlMapListeners = ({ view, mapInstance }: UseOlMapListenersInput): void => {
  const mapSize = useSelector(mapDataSizeSelector);
  const modelId = useSelector(currentModelIdSelector);
  const coordinate = useRef<Coordinate>([]);
  const pixel = useRef<Pixel>([]);
  const dispatch = useAppDispatch();

  const handleRightClick = useDebouncedCallback(
    onMapRightClick(mapSize, modelId, dispatch),
    OPTIONS.clickPersistTime,
    {
      leading: false,
    },
  );

  const handleChangeCenter = useDebouncedCallback(
    onMapPositionChange(mapSize, dispatch),
    OPTIONS.queryPersistTime,
    { leading: false },
  );

  const handleMapSingleClick = useDebouncedCallback(
    onMapSingleClick(mapSize, modelId, dispatch),
    OPTIONS.clickPersistTime,
    { leading: false },
  );

  useEffect(() => {
    const key = view.on('change:center', handleChangeCenter);

    return () => unByKey(key);
  }, [view, handleChangeCenter]);

  useEffect(() => {
    if (!mapInstance) {
      return;
    }

    const key = mapInstance.on('singleclick', handleMapSingleClick);

    // eslint-disable-next-line consistent-return
    return () => unByKey(key);
  }, [mapInstance, handleMapSingleClick]);

  useEffect(() => {
    if (!mapInstance) {
      return;
    }

    const rightClickEvent = (e: MouseEvent): Promise<void> | undefined => {
      e.preventDefault();

      coordinate.current = mapInstance.getEventCoordinate(e);
      pixel.current = mapInstance.getEventPixel(e);

      return handleRightClick(coordinate.current, pixel.current);
    };

    mapInstance.getViewport().addEventListener('contextmenu', rightClickEvent);

    // eslint-disable-next-line consistent-return
    return () => mapInstance.getViewport().removeEventListener('contextmenu', rightClickEvent);
  }, [mapInstance, handleRightClick]);
};
